from setux.core.package import CommonPackager


# pylint: disable=no-member


class Distro(CommonPackager):
    manager = 'npm'

    def do_init(self):
        self.target.distro.Package.install('npm')

    def do_installed(self):
        ret, out, err = self.run('npm list -g --depth=0')
        for line in out[1:]:
            if len(line)<5: continue
            n, v = line[4:].split('@')
            yield n, v

    def do_installable(self, pattern):
        ret, out, err = self.run(f'npm search {pattern}')
        for line in out[1:]:
            fields = line.split('|')
            name, ver = fields[0], fields[4]
            yield name.strip(), ver.strip()

    def do_remove(self, pkg):
        self.run(f'npm uninstall {pkg}')

    def do_cleanup(self):
        raise NotImplemented

    def do_update(self):
        raise NotImplemented

    def do_install(self, pkg, ver=None):
        self.run(f'npm install -g {pkg}')



from pybrary.net import get_ip_adr
from pybrary.func import memo

from setux.core.manage import Manager


class Distro(Manager):
    manager = 'system'

    @property
    def hostname(self):
        attr = '_hostname_'
        try:
            val = getattr(self, attr)
        except AttributeError:
            ret, out, err = self.run('hostname')
            val = out[0]
            setattr(self, attr,  val)
        return val

    @hostname.setter
    def hostname(self, val):
        attr = '_hostname_'
        delattr(self, attr)
        ret, out, err = self.run(
            'hostname', val.replace('_', '')
        )
        return ret

    @memo
    def fqdn(self):
        ret, out, err = self.run('hostname -f')
        return out[0]



from re import compile

from setux.logger import error, info
from setux.core.package import CommonPackager
from setux.targets import Local


# pylint: disable=no-member


class Distro(CommonPackager):
    manager = 'pip'

    def do_init(self):
        self.target.distro.Package.install('setuptools')
        self.run(f'python3 -m pip install -qU pip')

    def do_install(self, pkg, ver=None):
        ret, out, err = self.run(f'python3 -m pip install -qU {pkg}')
        for o in out:
            if 'already satisfied' in o:
                break
            if 'Successfully installed' in o:
                info('\t--> %s %s', pkg, ver or '')
                break
        else:
            if any(line.strip() for line in out):
                error('\n'.join(out))

    def do_installed(self):
        ret, out, err = self.run('python3 -m pip list')
        for line in out[2:]:
            try:
                n, v = line.split()
                v = v.replace('(', '')
                v = v.replace(')', '')
                yield n, v
            except:
                error(line)

    def get_pkg_list(self):
        local = Local(outdir='/tmp')
        dst = '/tmp/setux/pypi.packages'
        try:
            fil = local.file(dst)
            if fil.age < 10: return
        except FileNotFoundError: pass

        url = 'https://pypi.org/simple'
        org = '/tmp/setux/pypi.simple'
        local.dir('/tmp/setux').deploy()
        local.download(url=url, dst=org)
        pat = compile(r'^.*?<a href="/simple/.*?/">(?P<name>.*?)</a>')
        with open(dst, 'w') as out:
            for line in open(org):
                try:
                    name = pat.search(line).groupdict()['name']
                    out.write(f'{name}\n')
                except AttributeError: pass
        local.file(org).rm()

    def do_installable(self, pattern):
        dst = '/tmp/setux/pypi.packages'
        self.get_pkg_list()
        for name in open(dst):
            yield name.strip(), '-'

    def do_remove(self, pkg):
        self.run(f'python3 -m pip uninstall -y {pkg}')

    def do_cleanup(self):
        raise NotImplemented

    def do_update(self):
        raise NotImplemented




from setux.logger import debug
from setux.core.manage import SpecChecker


class Distro(SpecChecker):
    '''
        No Login User
    '''
    manager = 'user_'

    def do_validate(self, specs):
        yield 'home', f'/home/{self.key}' # useradd -M doesn't work
        yield 'shell', '/bin/false'
        for k, v in specs.items():
            if k=='uid':
                yield k, int(v)
            elif k=='gid':
                yield k, int(v)
            elif k=='home':
                debug('No Home for System User')
            elif k=='shell':
                debug('No Shell for System User')
            else:
                debug(f'Invalid Spec : {k}={v}')

    def get(self):
        user = self.key if self.key else self.spec['uid']
        ret, out, err = self.run(f'grep ^{user}: /etc/passwd', shell=True)
        for line in out:
            name, x, uid, gid, rem, home, shell = line.split(':')
            if self.key:
                if name != self.key: continue
            else:
                if int(uid) != self.spec['uid']: continue
            return dict(
                name = name,
                uid = int(uid),
                gid = int(gid),
                home = home,
                shell = shell,
            )

    @property
    def group(self):
        return self.distro.group(
            self.key,
            gid = self.spec.get('gid')
        )

    def cre(self, check=True):
        debug(f'user create {self.key}')
        self.do_cre(user=self.key, check=check)

    def mod(self, key, val, check=True):
        debug(f'NO MOD for System User {self.key} ({key} : {val})')

    def rm(self, check=True):
        debug(f'user delete {self.key}')
        self.do_rm(user=self.key, check=check)

    def do_cre(self, *, user, check):
        self.run(f'useradd -M {user}', check=check) # M doesn't work
        self.run(f'usermod -L {user}', check=check)
        self.run(f'usermod -s /bin/false {user}', check=check)
        self.target.deploy('sudoers', user=user)

    def do_rm(self, *, user, check):
        self.run(f'userdel -r {user}', check=check)


#__to__do__
class FreeBSD(Distro):
    opts = dict(
        uid   = 'u',
        gid   = 'g',
        shell = 's',
        home  = 'd',
    )

    def do_cre(self, *, user, check):
        self.run(f'pw useradd -n {user}', check=check)

    def do_mod(self, *, user, key, val, check):
        opt = self.opts[key]
        self.run(f'pw usermod -n {user} -{opt} {val}', check=check)

    def do_rm(self, *, user, check):
        self.run(f'pw userdel -n {user} -r', check=check)




from pybrary.net import get_ip_adr
from pybrary.func import memo

from setux.core.manage import Manager


class Distro(Manager):
    manager = 'net'

    @memo
    def addr(self):
        ok, adr = get_ip_adr()
        if ok:
            return adr
        else:
            debug(adr)
            return '!'

